#==============================================================================
#  Pull from an XLS calculate and return grades and comments as PDFs
#==============================================================================

# There are two files that are a part of generating the PDF.
# 1) This file that does all the data cleaning, merging, calculating and finally creating the PDF.
# 2) A .Rmd file that is the template for the PDF.

#==============================================================================
#   1. Settings
#==============================================================================
# Clear the working space
rm(list = ls(all = TRUE))

#==============================================================================
#   2. Packages
#==============================================================================

# install.packages("tidyverse")
library(tidyverse)
library(readxl)

# install.packages("knitr")
library(knitr)

#==============================================================================
#   2. Read in Excel Files
#==============================================================================
# Set the XLS file name to make it easier to reuse script
xlsPath <- "data/sample_grade_sheets.xlsx"


# Look at the sheet names if the excel file has multiple sheets.
excel_sheets(path = xlsPath)


# You can read in one sheet from an excel file by giving the sheet name as an argument
#       *Note reading in the `debates grade` sheet gives a warning about "New names:"
#               This is because there is an empty unnamed column on the `debates grade` sheet.
#               A new name is made up automatically for that column that sticks out.
#               It's not an issue we will just drop it later.
brief <- read_excel(path = xlsPath, sheet = "brief")
debate <- read_excel(path = xlsPath, sheet = "debate grade")
attendance <- read_excel(path = xlsPath, sheet = "attendance")


#==============================================================================
#   3. Merge all info into one tibble/data frame
#==============================================================================
# Before merging combine the last_name and first_name in `brief` as `student`.
brief <- mutate(brief, student = paste0(last_name,", ", first_name))


# You can only merge two tibbles/data frames at a time so I do this in two steps:
        # Merge `brief` and `debate` into `allGrades`
        allGrades <- full_join(x = brief, y = debate, by="student")

        # Merge `debate` into the new `allGrades`
        allGrades <- full_join(x = allGrades, y = attendance, by="student")


# Remove any empty rows by removing rows that do have a value for `group`.
#       I would normally have used `student`, but there is a note in that column on the attendance sheet.
#       This will also remove that comment.
allGrades <- filter(allGrades, is.na(group)==FALSE)


#==============================================================================
#   4. Calculations and new vars
#==============================================================================

# Add the `Different from others` variable.
allGrades <- mutate(allGrades, `Different (2)` = 2)

# Find the overall presentation grade
allGrades <- mutate(allGrades, `Presentaton Grade` = `Knowledge (10)` + `Persuade (10)` + `clarity (10)` +  `Different (2)`)

# Find the attendance grade
allGrades <- mutate(allGrades, `Attendance Grade` = (`Debate 1` + `Debate 2` + `Debate 3` + `Debate 4`)*2)

# Find the overall grades
allGrades <- mutate(allGrades, `Overall Grade` = `brief grade` + `Presentaton Grade` + `Q&A` + `Attendance Grade`)


#==============================================================================
#   5. Clean-Up
#==============================================================================

# Keep only the variables we need and reorganize order
keepVars <- c("student",
              "group",
              "brief grade",
              "Comments",
              "time",
              "Knowledge (10)",
              "Persuade (10)",
              "clarity (10)",
              "Different (2)",
              "Presentaton Grade",
              "Presentation Comments",
              "Q&A",
              "Q&A Comments",
              "Debate 1",
              "Debate 2",
              "Debate 3",
              "Debate 4",
              "Attendance Grade",
              "Overall Grade"
              )

allGrades <- select(allGrades, all_of(keepVars))


# Write out a CSV file as backup
# write_csv(x = allGrades, file = "data/overallGradesAndBreakdown.csv")


#==============================================================================
#   5. Create the PDFs
#==============================================================================

## This loops over each student for the assignment1 object we read in before.
for(i in 1:nrow(allGrades)){

        # Get student i's name in a form suitable for a file name
        studentNameForFileName <- str_replace(string = allGrades$student[i], pattern = ", ", replacement = "_")

        # This is the code that makes the PDF using RMarkdown.
        #       It takes the data for a student form the clean tibble/ data frame and maps it to a `param` name.
        #       R markdown uses those param names to fill in the blanks in the template.
        rmarkdown::render("gradeTemplateFederalist.Rmd",
                          params = list(
                                  studentName = allGrades$student[i],
                                  studentGroup = allGrades$group[i],
                                  briefGrade=allGrades$`brief grade`[i],
                                  briefComments=allGrades$Comments[i],
                                  presentationTime=allGrades$time[i],
                                  presentationKnowledge=allGrades$`Knowledge (10)`[i],
                                  presentationPersuade=allGrades$`Persuade (10)`[i],
                                  presentationClarity=allGrades$`clarity (10)`[i],
                                  presentationDifferent=allGrades$`Different (2)`[i],
                                  presentationGrade=allGrades$`Presentaton Grade`[i],
                                  presentationComments=allGrades$`Presentation Comments`[i],
                                  qAndAGrade=allGrades$`Q&A`[i],
                                  qAndAComments=allGrades$`Q&A Comments`[i],
                                  attendanceGrade=allGrades$`Attendance Grade`[i],
                                  overallGrade=allGrades$`Overall Grade`[i]
                                  ),
                          output_file = paste0("pdfs/",studentNameForFileName,"_GradeReport.pdf")
                          )

}

